package com.example.xmen_tp5_2324;

import static com.example.xmen_tp5_2324.databinding.ActivityMainBinding.inflate;
import androidx.activity.result.ActivityResult;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.Menu;

import com.example.xmen_tp5_2324.databinding.ActivityMainBinding;

import java.util.*;

public class MainActivity extends AppCompatActivity implements XMenAdapter.OnItemClickListener {
    private ActivityMainBinding ui;
    private ActivityResultLauncher<Intent> editLauncher;
    private List<XMen> liste;    // variable membre
    private XMenAdapter adapter; // variable membre

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //setContentView(R.layout.activity_main);
        ui = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(ui.getRoot());

        // obtenir la liste
        XMenApplication application = (XMenApplication) getApplication();
        liste = application.getListe();

        // créer l'adaptateur
        adapter = new XMenAdapter(liste);
        // on fournit l'activité comme écouteur de clics à l'adaptateur (qui le fournira lui-même aux view holders)
        adapter.setOnItemClickListener(this);

        // fournir l'adaptateur au recycler
        ui.recycler.setAdapter(adapter);
        ui.recycler.setHasFixedSize(true);

        RecyclerView.LayoutManager lm = new LinearLayoutManager(this);
        ui.recycler.setLayoutManager(lm);

        // création du launcher d'activité d'édition de personnage
        editLauncher = registerForActivityResult(
                new ActivityResultContracts.StartActivityForResult(),
                this::onEditActivityFinished);

    }

    /**
     * Cette méthode est appelée quand on revient dans cette activité
     * après l'avoir lancée par editLauncher
     */
   // @SuppressLint("NotifyDataSetChanged")
    @SuppressLint("NonConstantResourceId")
    private void onEditActivityFinished(ActivityResult result) {
        if (result.getResultCode() == RESULT_OK) {
            adapter.notifyDataSetChanged();
        }
    }

    @Override
    public void onItemClick(int position) {
        // récupérer le X-Men concerné
        XMen xmen = liste.get(position);
        // changer l'image du X-Men
        xmen.setIdImage(R.drawable.undef);
        // signaler à l'adaptateur que l'élément a changé
        adapter.notifyItemChanged(position);
    }
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @SuppressLint("NonConstantResourceId")
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        XMenApplication application = (XMenApplication) getApplication();
        switch (item.getItemId()) {
            case R.id.reinit:
                application.initListe();
                adapter.notifyDataSetChanged();
                return true;
            case R.id.create:
                // TODO voir exercice suivant
                onEdit(-1);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }



    /**
     * lancer l'édition sur un XMen indiqué par position, -1 si création
     * @param position: index dans la liste, mettre -1 si on veut créer un X-Men
     */
    private void onEdit(int position) {
        // créer un Intent pour lancer EditActivity
        Intent intent = new Intent(this,EditActivity.class);
        intent.putExtra("position",position);
        // démarrer l'activité avec attente du résultat
        editLauncher.launch(intent);
    }
    private void onDelete(int position) {
        // créer un Intent pour lancer EditActivity
        liste.remove(position);
        adapter.notifyDataSetChanged();
    }
    @Override
    public boolean onContextItemSelected(MenuItem item){
        // récupérer la position de l'élément concerné par le menu (dans order)
        int position = item.getOrder();

        // selon le bouton de menu
        switch (item.getItemId()) {
            case XMenViewHolder.MENU_EDIT:
                onEdit(position);
                return true;
            case XMenViewHolder.MENU_DELETE:
                // TODO décommenter quand cette méthode sera ajoutée, voir la fin du sujet
                 onDelete(position);
                return true;
        }
        return super.onContextItemSelected(item);
    }


}